@@ -36,9 +36,13 @@ microtable <- R6Class(classname = "microtable",
 		#' dataset$tidy_dataset()
 		initialize = function(otu_table = NULL, sample_table = NULL, tax_table = NULL, phylo_tree = NULL)
 			{
-			self$otu_table <- otu_table
+			if(!all(sapply(otu_table, is.numeric))){
+				stop("Some columns in otu_table are not numeric vector! Please check the otu_table and try again.")
+			}else{
+				self$otu_table <- otu_table			
+			}
 			if(is.null(sample_table)){
-				message("No sample_table provided, automatically use colnames of otu_table to create one.")
+				message("No sample_table provided, automatically use colnames of otu_table to create it.")
 				self$sample_table <- data.frame(SampleID = colnames(otu_table), Group = colnames(otu_table)) %>% `row.names<-`(.$SampleID)
 			}else{
 				self$sample_table <- sample_table
@@ -132,6 +136,9 @@ microtable <- R6Class(classname = "microtable",
 		#' dataset$tidy_dataset(main_data = TRUE)
 		tidy_dataset = function(main_data = TRUE){
 			sample_names <- intersect(rownames(self$sample_table), colnames(self$otu_table))
+			if(length(sample_names) == 0){
+				stop("No same sample name found between rownames of sample_table and colnames of otu_table! Please check whether the rownames of sample_table are sample names!")
+			}
 			# keep the sample order same with raw sample table
 			sample_names <- rownames(self$sample_table) %>% .[. %in% sample_names]
 			self$sample_table %<>% .[sample_names, , drop = FALSE]
@@ -491,5 +498,3 @@ microtable <- R6Class(classname = "microtable",
 	lock_objects = FALSE,
 	lock_class = FALSE
 )
-
-
